@@ -3,7 +3,8 @@
 
    This file is part of LibTMCG.
 
- Copyright (C) 2004, 2005, 2007, 2016--2018  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2004, 2005, 2007,
+               2016, 2017, 2018, 2019  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,12 +39,14 @@ static const std::string LibTMCG_ID =
 void* tmcg_realloc
 	(void* ptr, size_t old_size, size_t new_size)
 {
+	assert((old_size == 0) || (old_size != 0)); // to supress compiler warning
 	return gcry_realloc(ptr, new_size);
 }
 void tmcg_free
 	(void* ptr, size_t size)
 {
-	gcry_free(ptr);
+	if (size > 0)
+		gcry_free(ptr);
 }
 
 // LibTMCG general functions
